#include "OperationModeToggler.h"
#include "DeviceManager.h"
#include "StartupSettings.h"
#include "AlarmSender.h"
#include "DeviceConfigurator.h"
#include "DeviceDataStorage.h"

#include <QtDebug>

OperationModeToggler::OperationModeToggler( 
		DeviceManager::OperationMode aMode, 
		StartupSettings *aSettings, 
		AlarmSender *aAlarmSender, 
		DeviceConfigurator *aConfigurator,
		DeviceDataStorage *aDataStorage
		) :
	iMode( aMode ),
	iSettings( aSettings ),
	iAlarmSender( aAlarmSender ),
	iConfigurator( aConfigurator ),
	iDataStorage( aDataStorage ),
	iSuccess( true )
{
	qDebug() << "OperationModeToggler::OperationModeToggler( ... )";
	
	qRegisterMetaType<DeviceManager::ErrorCode>("DeviceManager::ErrorCode");
	
	connect( iAlarmSender, SIGNAL( alarmSendingFailed( DeviceManager::ErrorCode, const QString& ) ),
			this, SIGNAL( error( DeviceManager::ErrorCode, const QString& ) ) );
	connect( iConfigurator, SIGNAL( configuringError( DeviceManager::ErrorCode ) ),
			this, SLOT( createError( DeviceManager::ErrorCode ) ) );
	connect( iDataStorage, SIGNAL( dataStorageInitFailed( DeviceManager::ErrorCode ) ),
			this, SLOT( createError( DeviceManager::ErrorCode ) ) );
}

OperationModeToggler::~OperationModeToggler()
{
	qDebug() << "OperationModeToggler::~OperationModeToggler()";
}

void OperationModeToggler::run()
{
	switch ( iMode )
	{
		case DeviceManager::EmptyMode:
			// error occured. Mode cannot be changed
			createError( DeviceManager::ModeNotFetched );
			iSuccess = false;
			return;
		case DeviceManager::StandAloneMode:
			
			// change to KioskMode
			
			// check if auto turn on/off functionality enabled and send turn on/off alarm events to alarm daemon
			if ( iSettings->isPowersavingEnabled() )
			{
				emit changingMode( "Sending the auto launch alarms to alarm daemon." );
				if ( !iAlarmSender->sendAlarms( iSettings->turnOnAt(), iSettings->turnOffAt() ) )
				{
					iSuccess = false;
					return;	//this is critical so returning if no success
				}
			}
			
			// - disable the certain hw keys (only "home"-hw-key at the moment)
			// - register init script to launch the application when ever the device is launched
			// - disable the screen "auto-switch-off" and "dimming"
			// - store info about the new operation mode
			
			emit changingMode( "Disabling home hardware-key." );
			sleep( 2 );
			if( !iConfigurator->toggleHWKeys( false ) )
				iSuccess = false;
			if( iSuccess ) {
				emit changingMode( "Installing the application init start-up script." );
				sleep( 2 );
				if ( !iConfigurator->toggleInitScript( true ) )
					iSuccess = false;
			}
			if( iSuccess ) {
				emit changingMode( "Disabling the screen switching off and dimming." );
				sleep( 2 );
				if( !iConfigurator->toggleScreenSwitchOff( false ) )
					iSuccess = false;
			}
			if( iSuccess ) {
				emit changingMode( "Storing information about the new operation mode." );
				sleep( 2 );
				if( !storeOperationMode( DeviceManager::KioskMode, iDataStorage ) ) {
					createError( DeviceManager::ModeNotStored );
					iSuccess = false;
				}	
			}
			if( !iSuccess ) {
				// we have to roll back if something fails
				// of course rolling back may fail as well but it is impossible to catch
				emit toggleErrorSending( false );
				iAlarmSender->removeAlarms();
				iConfigurator->toggleHWKeys( true );
				iConfigurator->toggleInitScript( false );
				iConfigurator->toggleScreenSwitchOff( true );
				emit toggleErrorSending( true );
				return;
			}
			break;

		case DeviceManager::KioskMode:
			// change to StandAloneInProgress mode

			// - enable the certain hw keys (only "home"-hw-key at the moment)
			// - unregister the init script
			// - enable the screen "auto-switch-off" and "dimming"
			// - store info about the new operation mode
			emit changingMode( "Enabling home hardware-key." );
			sleep( 2 );
			if( !iConfigurator->toggleHWKeys( true ) )
				iSuccess = false;
			if( iSuccess ) {
				emit changingMode( "Enabling the screen switching off and dimming." );
				sleep( 2 );
				if( !iConfigurator->toggleScreenSwitchOff( true ) )
					iSuccess = false;
			}
			if( iSuccess ) {
				emit changingMode( "Storing information about the new operation mode." );
				sleep( 2 );
				if( !storeOperationMode( DeviceManager::StandAloneModeInProgress, iDataStorage ) ) {
					createError( DeviceManager::ModeNotStored );
					iSuccess = false;
				}
			}
			if( iSuccess ) {
				emit changingMode( "Removing the auto launch alarms from alarm daemon." );
				sleep( 2 );
				if( !iAlarmSender->removeStoredAlarms() )
					iSuccess = false;
			}
			if( !iSuccess ) {
				// we have to roll back if something fails
				// of course rolling back may fail as well but it is impossible to catch
				emit toggleErrorSending( false );
				iConfigurator->toggleHWKeys( false );
				iConfigurator->toggleInitScript( true );
				iConfigurator->toggleScreenSwitchOff( false );
				storeOperationMode( DeviceManager::KioskMode, iDataStorage );
				emit toggleErrorSending( true );
				return;
			}
			break;
		default: // StandAloneModeInProgress should never come in question
			break;
	}
}

bool OperationModeToggler::success()
{
	qDebug() << "OperationModeToggler::success()";
	return iSuccess;
}
			
bool OperationModeToggler::storeOperationMode( DeviceManager::OperationMode aMode, DeviceDataStorage *aDataStorage )
{
	qDebug() << "OperationModeToggler::storeOperationMode( const OperationMode & )";
	QStringList modeStrList;
	QString str;
	modeStrList.append( str.append( QString( "%1" ).arg( aMode ) ) );
	if ( !aDataStorage->storeData( aDataStorage->dataSectionToString( DeviceDataStorage::DeviceMode ), modeStrList ) )
		return false;
	return true;
}

void OperationModeToggler::createError( DeviceManager::ErrorCode aCode )
{
	qDebug() << "OperationModeToggler::createError( DeviceManager::ErrorCode )";
	QString empty = "";
	emit error( aCode, empty );
}

