#include "WindowManager.h"

#include <QEvent>
#include <QDialog>
#include "ViewBase.h"
#include "PopUpMessageBox.h"

#include <QtDebug>

WindowManager::WindowManager( QWidget *aParent ) :
		QWidget( aParent ),
		iApplicationName( tr( "Qt Meetings" ) ),
		iCurrentView( 0 )
{
	this->setWindowTitle( iApplicationName );
}

WindowManager::~WindowManager()
{
	
}

void WindowManager::showView( ViewBase *view )
{
	// The views parent must be WindowManager when it is displayed trough this
	QWidget *parent = static_cast<QWidget *>(view->parent());
	if ( parent != this )
	{
		view->setParent( this );
	}
	
	// Store the current view because it is hidden after the new view is shown
	QWidget *oldView = iCurrentView;
	
	// If the new view is observed view we store the current into stack
	// from which it is restored when the new view receives event we are
	// listening to.
	if ( view->viewMode() == ViewBase::ObservedView )
	{
		iViewList.push( iCurrentView );
	}
	
	// Make the new view visible and handle connections
	iCurrentView = view;
	connect( iCurrentView, SIGNAL( eventDetected() ), this, SLOT( viewEventDetected() ) );
	connect( this, SIGNAL( viewResized(const QSize &, const QSize &) ), iCurrentView, SLOT( viewResized( const QSize &, const QSize & ) ) );
	view->resize( this->size() );
	
	view->show();
	
	// Disconnect old connections and hide the view
	if ( oldView != 0 )
	{
		disconnect( oldView, SIGNAL( eventDetected() ), this, SLOT( viewEventDetected() ) );
		disconnect( this, SIGNAL( viewResized(const QSize &, const QSize &) ), oldView, SLOT( viewResized(const QSize &, const QSize &) ) );
		oldView->hide();
	}
	
}

void WindowManager::showDialog(QDialog *aDialog, bool blocking, bool aSendSignal)
{
	// Handle dialog displaying
	if ( aSendSignal ) emit dialogActivated();
	if ( blocking )
	{
		aDialog->exec();
	}
	else
	{
		aDialog->show();
	}
	if ( aSendSignal ) emit dialogDeactivated();
}

void WindowManager::viewEventDetected()
{
	
	if ( iCurrentView != 0 )
	{
		if ( iCurrentView->viewMode() == ViewBase::NormalView )
		{
			emit eventDetected();
		}
		else if ( iCurrentView->viewMode() == ViewBase::ObservedView )
		{
			if ( !iViewList.isEmpty() )
			{
				ViewBase *previousView = static_cast<ViewBase *>( iViewList.pop() );
				this->showView( previousView );
				emit previousViewRestored();
			}
		}
	}

}

bool WindowManager::event(QEvent *event)
{
	if ( event->type() == QEvent::Resize )
	{
		if ( iCurrentView != 0 )
		{
			QSize currentSize = iCurrentView->size();
			iCurrentView->setFixedSize( this->size() );
			emit viewResized( this->size(), currentSize );
		}
	}
	
	return QWidget::event( event );
}

void WindowManager::error( const QString &aErrorMessage )
{
	qDebug() << "WindowManager::error	";

	PopUpMessageBox *popup = PopUpMessageBox::error( 0, aErrorMessage );
	if ( popup != 0 )
	{
		showDialog( static_cast<QDialog *>( popup ), false );
	}
}

void WindowManager::setFullscreen()
{
	this->setWindowState( Qt::WindowFullScreen );
	// Resize event handles the rest.
}

void WindowManager::connectionLost()
{
	//TODO: Inform all views
	qDebug() << "WindowManager::connectionLost()";
}

void WindowManager::connectionEstablished()
{
	//TODO: Inform all views
	qDebug() << "WindowManager::connectionEstablished()";
}

