#ifndef WINDOWMANAGER_H_
#define WINDOWMANAGER_H_

#include <QWidget>
#include <QStack>

// Forward declarations
class ViewBase;
class QEvent;
class QSize;
class QDialog;
class QString;

//! UserInterface class. Manages displayed views.
/*!
 * UserInterface class. WindowManager class is responsible for displaying views that inherit the
 * ViewBase class. It also handles dialog showing. Depending on the views type the WindowManager
 * can track the views events and restore previous view if the current on is ObservedView. This
 * is a handy mechanism for screensaver etc.
 */
class WindowManager : public QWidget
{
	Q_OBJECT

public:
	//! Constructor.
	/*!
	 * Constructor of WindowManager.
	 */
	WindowManager( QWidget *aParent = 0 );
	//! Destructor.
	virtual ~WindowManager();
	
	virtual bool event(QEvent *event);

	
	//! Shows any view specific indicators for connection error
	void connectionLost();
	
	//! Removes any view specific indicators for connection error
	void connectionEstablished();

signals:
	//! Request current status of the room.
	/*!
	 * Signal is emitted when there is need to check current status of room aRoom.
	 * \param aRoom Meetingroom which status is requested.
	 */
	void eventDetected();
	
	//! The view size is changed.
	/*!
	 * This signal is emitted when the window managers view changes,
	 * i.e. it received resized QEvent.
	 * \param The new view size.
	 */
	void viewResized(const QSize &newSize, const QSize &oldSize);
	
	//! Previous view is restored.
	/*!
	 * This signal is emitted when previously stored view is
	 * restored. This happens when view with type ViewMode::ObservedView
	 * is shown and it receives an event that initiates the view
	 * restoring chain.
	 */
	void previousViewRestored();
	
	void dialogActivated();
	void dialogDeactivated();

public slots:
	//! Shows the view.
	/*!
	 * Show view that inherits ViewBase class. If the views parent is not
	 * the WindowManager it will changed within this method. Depeding on the
	 * views type the currently active view might be stored and restored
	 * when specific event occurs in the view to be displayed.
	 */
	void showView( ViewBase *view );
	
	//! Shows modal dialog.
	/*!
	 * Shows modal dialog. Emits dialogActivated() signal prior calling
	 * QDialog's exec() method and emits dialogDeactivated signal when
	 * the exec() method returns.
	 */
	void showDialog( QDialog *aDialog, bool blocking = true, bool aSendSignal = true );
	
	//! View event is detected.
	/*!
	 * WindowManager connects this slot to ViewBase classes eventDetected()
	 * signal and either emits eventDetected() signal if the current views
	 * type is ViewMode::NormalView or restores possible previous view
	 * if the current views type is ViewMode::ObservedView.
	 */
	void viewEventDetected();
	
	void setFullscreen();
	
	void error( const QString &aErrorMessage );

private:
	//! Name of the application.
	QString iApplicationName;
	
	//! Currently active view.
	ViewBase *iCurrentView;
	
	//! Stack of views previously displayed.
	QStack<ViewBase *> iViewList;

};

#endif /*WINDOWMANAGER_H_*/
